@@ -2,7 +2,7 @@
 #
 #       clipboard.py
 #
-#       Copyright 2009-2016 Giuseppe Penone <giuspen@gmail.com>
+#       Copyright 2009-2019 Giuseppe Penone <giuspen@gmail.com>
 #
 #       This program is free software; you can redistribute it and/or modify
 #       it under the terms of the GNU General Public License as published by
@@ -19,8 +19,17 @@
 #       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 #       MA 02110-1301, USA.
 
-import gtk, os, xml.dom.minidom, re, base64, mimetypes
-import cons, machines, exports, imports, support
+import gtk
+import os
+import xml.dom.minidom
+import re
+import base64
+import mimetypes
+import cons
+import machines
+import exports
+import imports
+import support
 
 
 TARGET_CTD_PLAIN_TEXT = 'UTF8_STRING'
@@ -120,7 +129,7 @@ class ClipboardHandler:
         self.clipboard.set_with_data([(t, 0, 0) for t in (TARGET_CTD_TABLE, TARGETS_HTML[0])],
                                      self.get_func,
                                      self.clear_func,
-                                     (table_dict, None, html_text, None))
+                                     (None, None, html_text, table_dict))
 
     def table_row_paste(self, model_n_iter):
         """Paste Table Row from the Clipboard"""
@@ -143,21 +152,26 @@ class ClipboardHandler:
                 elif "liststore" in anchor_dir:
                     table_dict = self.dad.state_machine.table_to_dict(anchor)
                     html_text = self.dad.html_handler.table_export_to_html(table_dict)
-                    self.clipboard.set_with_data([(t, 0, 0) for t in (TARGET_CTD_TABLE, TARGETS_HTML[0])],
+                    txt_handler = exports.Export2Txt(self.dad)
+                    text_offsets_range = [iter_sel_start.get_offset(), iter_sel_end.get_offset()]
+                    plain_text = txt_handler.node_export_to_txt(text_buffer, "", sel_range=text_offsets_range, check_link_target=True)
+                    self.clipboard.set_with_data([(t, 0, 0) for t in (TARGET_CTD_TABLE, TARGETS_HTML[0], TARGET_CTD_PLAIN_TEXT)],
                                                  self.get_func,
                                                  self.clear_func,
-                                                 (table_dict, None, html_text, None))
+                                                 (plain_text, None, html_text, table_dict))
                     return
                 elif "sourcebuffer" in anchor_dir:
                     codebox_dict = self.dad.state_machine.codebox_to_dict(anchor, for_print=0)
                     codebox_dict_html = self.dad.state_machine.codebox_to_dict(anchor, for_print=2)
                     html_text = self.dad.html_handler.codebox_export_to_html(codebox_dict_html)
-                    self.clipboard.set_with_data([(t, 0, 0) for t in (TARGET_CTD_CODEBOX, TARGETS_HTML[0])],
+                    txt_handler = exports.Export2Txt(self.dad)
+                    text_offsets_range = [iter_sel_start.get_offset(), iter_sel_end.get_offset()]
+                    plain_text = txt_handler.node_export_to_txt(text_buffer, "", sel_range=text_offsets_range, check_link_target=True)
+                    self.clipboard.set_with_data([(t, 0, 0) for t in (TARGET_CTD_CODEBOX, TARGETS_HTML[0], TARGET_CTD_PLAIN_TEXT)],
                                                  self.get_func,
                                                  self.clear_func,
-                                                 (codebox_dict, None, html_text, None))
+                                                 (plain_text, None, html_text, codebox_dict))
                     return
-        if not os.path.isdir(cons.TMP_FOLDER): os.mkdir(cons.TMP_FOLDER)
         html_text = self.dad.html_handler.selection_export_to_html(text_buffer, iter_sel_start, iter_sel_end,
             self.dad.syntax_highlighting if not from_codebox else cons.PLAIN_TEXT_ID)
         if not from_codebox and self.dad.syntax_highlighting == cons.RICH_TEXT_ID:
@@ -202,11 +216,11 @@ class ClipboardHandler:
                     selectiondata.set(target, 8, Win32HtmlFormat.encode(data[2]))
         elif target == TARGET_CTD_CODEBOX:
             dom = xml.dom.minidom.Document()
-            self.dad.xml_handler.codebox_element_to_xml([0, data[0], cons.TAG_PROP_LEFT], dom, dom)
+            self.dad.xml_handler.codebox_element_to_xml([0, data[3], cons.TAG_PROP_LEFT], dom, dom)
             selectiondata.set('UTF8_STRING', 8, dom.toxml())
         elif target == TARGET_CTD_TABLE:
             dom = xml.dom.minidom.Document()
-            self.dad.xml_handler.table_element_to_xml([0, data[0], cons.TAG_PROP_LEFT], dom, dom)
+            self.dad.xml_handler.table_element_to_xml([0, data[3], cons.TAG_PROP_LEFT], dom, dom)
             selectiondata.set('UTF8_STRING', 8, dom.toxml())
         elif target == TARGETS_IMAGES[0]: selectiondata.set_pixbuf(data[3])
 
@@ -265,46 +279,47 @@ class ClipboardHandler:
         if self.dad.syntax_highlighting != cons.RICH_TEXT_ID:
             iter_insert = self.dad.curr_buffer.get_iter_at_mark(self.dad.curr_buffer.get_insert())
             self.dad.curr_buffer.insert(iter_insert, selection_data)
-            return
-        uri_list = selection_data.split(cons.CHAR_NEWLINE)
-        for element in uri_list:
-            if not element: continue
-            iter_insert = self.dad.curr_buffer.get_iter_at_mark(self.dad.curr_buffer.get_insert())
-            if support.get_first_chars_of_string_are(element, cons.WEB_LINK_STARTERS):
-                property_value = "webs " + element
-            elif element.startswith("file://"):
-                file_path = element[7:].replace("%20", cons.CHAR_SPACE)
-                mimetype = mimetypes.guess_type(file_path)[0]
-                if mimetype and mimetype.startswith("image/") and os.path.isfile(file_path):
-                    try:
-                        pixbuf = gtk.gdk.pixbuf_new_from_file(file_path)
-                        self.dad.image_insert(iter_insert, pixbuf)
-                        iter_insert = self.dad.curr_buffer.get_iter_at_mark(self.dad.curr_buffer.get_insert())
-                        self.dad.curr_buffer.insert(iter_insert, 3*cons.CHAR_SPACE)
-                        continue
-                    except: pass
-                if os.path.isdir(file_path):
-                    property_value = "fold %s" % base64.b64encode(file_path)
-                elif os.path.isfile(file_path):
-                    property_value = "file %s" % base64.b64encode(file_path)
+        else:
+            uri_list = selection_data.split(cons.CHAR_NEWLINE)
+            for element in uri_list:
+                if not element: continue
+                iter_insert = self.dad.curr_buffer.get_iter_at_mark(self.dad.curr_buffer.get_insert())
+                if support.get_first_chars_of_string_are(element, cons.WEB_LINK_STARTERS):
+                    property_value = "webs " + element
+                elif element.startswith("file://"):
+                    file_path = element[7:].replace("%20", cons.CHAR_SPACE)
+                    mimetype = mimetypes.guess_type(file_path)[0]
+                    if mimetype and mimetype.startswith("image/") and os.path.isfile(file_path):
+                        try:
+                            pixbuf = gtk.gdk.pixbuf_new_from_file(file_path)
+                            self.dad.image_insert(iter_insert, pixbuf)
+                            iter_insert = self.dad.curr_buffer.get_iter_at_mark(self.dad.curr_buffer.get_insert())
+                            self.dad.curr_buffer.insert(iter_insert, 3*cons.CHAR_SPACE)
+                            continue
+                        except: pass
+                    if os.path.isdir(file_path):
+                        property_value = "fold %s" % base64.b64encode(file_path)
+                    elif os.path.isfile(file_path):
+                        property_value = "file %s" % base64.b64encode(file_path)
+                    else:
+                        property_value = None
+                        print "ERROR: discarded file uri '%s'" % file_path
                 else:
-                    property_value = None
-                    print "ERROR: discarded file uri '%s'" % file_path
-            else:
-                if os.path.isdir(element):
-                    property_value = "fold %s" % base64.b64encode(element)
-                elif os.path.isfile(element):
-                    property_value = "file %s" % base64.b64encode(element)
-                else:
-                    property_value = None
-                    print "ERROR: discarded ? uri '%s'" % element
-            start_offset = iter_insert.get_offset()
-            self.dad.curr_buffer.insert(iter_insert, element + cons.CHAR_NEWLINE)
-            if property_value:
-                iter_sel_start = self.dad.curr_buffer.get_iter_at_offset(start_offset)
-                iter_sel_end = self.dad.curr_buffer.get_iter_at_offset(start_offset + len(element))
-                self.dad.curr_buffer.apply_tag_by_name(self.dad.apply_tag_exist_or_create("link", property_value),
-                                                       iter_sel_start, iter_sel_end)
+                    if os.path.isdir(element):
+                        property_value = "fold %s" % base64.b64encode(element)
+                    elif os.path.isfile(element):
+                        property_value = "file %s" % base64.b64encode(element)
+                    else:
+                        property_value = None
+                        print "ERROR: discarded ? uri '%s'" % element
+                start_offset = iter_insert.get_offset()
+                self.dad.curr_buffer.insert(iter_insert, element + cons.CHAR_NEWLINE)
+                if property_value:
+                    iter_sel_start = self.dad.curr_buffer.get_iter_at_offset(start_offset)
+                    iter_sel_end = self.dad.curr_buffer.get_iter_at_offset(start_offset + len(element))
+                    self.dad.curr_buffer.apply_tag_by_name(self.dad.apply_tag_exist_or_create(cons.TAG_LINK, property_value),
+                                                           iter_sel_start, iter_sel_end)
+        self.dad.sourceview.scroll_mark_onscreen(self.dad.curr_buffer.get_insert())
 
     def to_html(self, clipboard, selectiondata, data):
         """From Clipboard to HTML Text"""
@@ -319,6 +334,7 @@ class ClipboardHandler:
         html_import = imports.HTMLHandler(self.dad)
         xml_string = html_import.get_clipboard_selection_xml(selection_data)
         self.from_xml_string_to_buffer(xml_string)
+        self.dad.sourceview.scroll_mark_onscreen(self.dad.curr_buffer.get_insert())
 
     def to_plain_text(self, clipboard, selectiondata, data):
         """From Clipboard to Plain Text"""
@@ -338,7 +354,7 @@ class ClipboardHandler:
                     link_url = plain_text[offsets[0]:offsets[1]]
                     if link_url[0:3] not in ["htt", "ftp"]: link_url = "http://" + link_url
                     property_value = "webs " + link_url
-                    self.dad.curr_buffer.apply_tag_by_name(self.dad.apply_tag_exist_or_create("link", property_value),
+                    self.dad.curr_buffer.apply_tag_by_name(self.dad.apply_tag_exist_or_create(cons.TAG_LINK, property_value),
                                                            iter_sel_start, iter_sel_end)
             else:
                 # check for file or folder path
@@ -352,9 +368,10 @@ class ClipboardHandler:
                         iter_sel_end = self.dad.curr_buffer.get_iter_at_mark(self.dad.curr_buffer.get_insert())
                         iter_sel_start = iter_sel_end.copy()
                         iter_sel_start.backward_chars(len(plain_text))
-                        self.dad.curr_buffer.apply_tag_by_name(self.dad.apply_tag_exist_or_create("link", property_value),
+                        self.dad.curr_buffer.apply_tag_by_name(self.dad.apply_tag_exist_or_create(cons.TAG_LINK, property_value),
                                                                iter_sel_start, iter_sel_end)
         self.force_plain_text = False
+        self.dad.sourceview.scroll_mark_onscreen(self.dad.curr_buffer.get_insert())
 
     def to_rich_text(self, clipboard, selectiondata, data):
         """From Clipboard to Rich Text"""
@@ -363,6 +380,7 @@ class ClipboardHandler:
             print "? no clipboard rich text"
             return
         self.from_xml_string_to_buffer(rich_text)
+        self.dad.sourceview.scroll_mark_onscreen(self.dad.curr_buffer.get_insert())
 
     def from_xml_string_to_buffer(self, xml_string):
         """From XML String to Text Buffer"""
@@ -371,6 +389,7 @@ class ClipboardHandler:
         if dom_node.nodeName != "root":
             print "rich text from clipboard error"
             return
+        self.dad.state_machine.not_undoable_timeslot_set(True)
         child_dom_iter = dom_node.firstChild
         while child_dom_iter != None:
             if child_dom_iter.nodeName == "slot":
@@ -386,6 +405,7 @@ class ClipboardHandler:
                         self.dom_node_to_codebox(nephew_dom_iter)
                     nephew_dom_iter = nephew_dom_iter.nextSibling
             child_dom_iter = child_dom_iter.nextSibling
+        self.dad.state_machine.not_undoable_timeslot_set(False)
 
     def dom_node_to_rich_text(self, dom_node):
         """From dom_node to Rich Text"""
@@ -410,6 +430,7 @@ class ClipboardHandler:
         pixbuf = selectiondata.get_pixbuf()
         pixbuf.link = ""
         self.dad.image_insert(self.dad.curr_buffer.get_iter_at_mark(self.dad.curr_buffer.get_insert()), pixbuf)
+        self.dad.sourceview.scroll_mark_onscreen(self.dad.curr_buffer.get_insert())
 
     def dom_node_to_image(self, dom_node):
         """From dom_node to Image"""
@@ -443,6 +464,7 @@ class ClipboardHandler:
             print "codebox from clipboard error"
             return
         self.dom_node_to_codebox(dom_node)
+        self.dad.sourceview.scroll_mark_onscreen(self.dad.curr_buffer.get_insert())
 
     def dom_node_to_codebox(self, dom_node):
         """From dom_node to CodeBox"""
@@ -473,6 +495,7 @@ class ClipboardHandler:
             print "table from clipboard error"
             return
         self.dom_node_to_table(dom_node, table_model_n_iter)
+        self.dad.sourceview.scroll_mark_onscreen(self.dad.curr_buffer.get_insert())
 
     def dom_node_to_table(self, dom_node, table_model_n_iter):
         """From dom_node to Table"""
