@@ -2,7 +2,7 @@
 #
 #       main.py
 #
-#       Copyright 2009-2016 Giuseppe Penone <giuspen@gmail.com>
+#       Copyright 2009-2019 Giuseppe Penone <giuspen@gmail.com>
 #
 #       This program is free software; you can redistribute it and/or modify
 #       it under the terms of the GNU General Public License as published by
@@ -19,32 +19,45 @@
 #       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 #       MA 02110-1301, USA.
 
-import gtk, gobject
-import dbus
-import dbus.service
-import dbus.mainloop.glib
-import sys, os, subprocess, gettext
+import gtk
+import gobject
+
+import sys
+import os
+import shutil
+import subprocess
+import gettext
 import __builtin__
+
+__builtin__.DBUS_OK = True
+try:
+    import dbus
+    import dbus.service
+    import dbus.mainloop.glib
+except:
+    __builtin__.DBUS_OK = False
+
 msg_server_to_core = {'f':0, 'p':""}
 __builtin__.msg_server_to_core = msg_server_to_core
 import cons, core
 
 
-class CherryTreeObject(dbus.service.Object):
+if __builtin__.DBUS_OK is True:
+    class CherryTreeObject(dbus.service.Object):
 
-    @dbus.service.method("com.giuspen.CherryTreeInterface",
-                         in_signature='s', out_signature='s')
-    def Send(self, in_message):
-        if len(in_message) < 4 or in_message[:4] != "ct*=":
-            print "bad in_message =", in_message
-            return ""
-        sep_pos = in_message.find("\x03")
-        filepath = filepath_fix(in_message[4:sep_pos] if sep_pos != -1 else in_message[4:])
-        node_name = in_message[sep_pos+1:] if sep_pos != -1 else ""
-        msg_server_to_core['p'] = filepath
-        msg_server_to_core['n'] = node_name
-        msg_server_to_core['f'] = 1
-        return "okz"
+        @dbus.service.method("com.giuspen.CherryTreeInterface",
+                             in_signature='s', out_signature='s')
+        def Send(self, in_message):
+            if len(in_message) < 4 or in_message[:4] != "ct*=":
+                print "bad in_message =", in_message
+                return ""
+            sep_pos = in_message.find("\x03")
+            filepath = in_message[4:sep_pos] if sep_pos != -1 else in_message[4:]
+            node_name = in_message[sep_pos+1:] if sep_pos != -1 else ""
+            msg_server_to_core['p'] = filepath
+            msg_server_to_core['n'] = node_name
+            msg_server_to_core['f'] = 1
+            return "okz"
 
 
 class CherryTreeHandler():
@@ -56,12 +69,18 @@ class CherryTreeHandler():
 
         if args.export_to_html_dir:
             if args.filepath:
-                ghost_window = core.CherryTree(self.lang_str, filepath_fix(args.filepath), args.node, self, True, True, True)
-                ghost_window.export_to_html("Auto", args.export_to_html_dir)
+                ghost_window = core.CherryTree(self.lang_str, args.filepath, args.node, self, True, True, True)
+                ghost_window.export_to_html("Auto", args.export_to_html_dir, args.export_overwrite)
+            else:
+                print "Export error: input not specified"
+        elif args.export_to_txt_dir:
+            if args.filepath:
+                ghost_window = core.CherryTree(self.lang_str, args.filepath, args.node, self, True, True, True)
+                ghost_window.export_to_txt_multiple("Auto", args.export_to_txt_dir, args.export_overwrite)
             else:
                 print "Export error: input not specified"
         else:
-            self.window_open_new(filepath_fix(args.filepath), args.node, True, True if args.filepath else False)
+            self.window_open_new(args.filepath, args.node, True, True if args.filepath else False)
             self.server_check_timer_id = gobject.timeout_add(1000, self.server_periodic_check) # 1 sec
 
     def window_open_new(self, filepath, node_name, is_startup, is_arg):
@@ -81,7 +100,10 @@ class CherryTreeHandler():
             print "win destroy: runn_win not found"
             i = -1
         self.running_windows.pop(i)
-        if not self.running_windows: gtk.main_quit()
+        if not self.running_windows:
+            gtk.main_quit()
+            try: shutil.rmtree(cons.TMP_FOLDER)
+            except: pass
 
     def server_periodic_check(self):
         """Check Whether the server posted messages"""
@@ -148,7 +170,8 @@ def initializations():
         except:
             print "libc.prctl not available, the process name will be python and not cherrytree"
     import locale
-    locale.setlocale(locale.LC_ALL, '')
+    try: locale.setlocale(locale.LC_ALL, '')
+    except: print "locale.setlocale failure"
     try:
         # change locale text domain
         locale.bindtextdomain(cons.APP_NAME, cons.LOCALE_PATH)
@@ -183,29 +206,33 @@ def initializations():
     return lang_str
 
 
-def filepath_fix(filepath):
+def arg_filepath_fix(filepath):
     """Fix a FilePath to an Absolute Path"""
     if not filepath: return ""
-    if not os.path.dirname(filepath): filepath = os.path.join(os.getcwd(), filepath)
-    else: filepath = os.path.abspath(filepath)
-    return filepath
+    filepath = filepath.decode(sys.getfilesystemencoding()).encode(cons.STR_UTF8, cons.STR_IGNORE)
+    if not os.path.dirname(filepath):
+        filepath = os.path.join(os.getcwd(), filepath)
+    else:
+        filepath = os.path.abspath(filepath)
+    return os.path.realpath(filepath)
 
 
 def main(args):
     """Everything Starts from Here"""
-    dbus.mainloop.glib.DBusGMainLoop(set_as_default=True)
-    try:
-        session_bus = dbus.SessionBus()
-        DBUS_OK = True
-    except:
-        DBUS_OK = False
+    if __builtin__.DBUS_OK is True:
+        dbus.mainloop.glib.DBusGMainLoop(set_as_default=True)
+        try:
+            session_bus = dbus.SessionBus()
+        except:
+            __builtin__.DBUS_OK = False
 
-    if args.filepath:
-        args.filepath = args.filepath.decode(sys.getfilesystemencoding()).encode(cons.STR_UTF8, cons.STR_IGNORE)
-    if args.export_to_html_dir:
+    args.filepath = arg_filepath_fix(args.filepath)
+    if args.export_to_html_dir or args.export_to_txt_dir:
+        args.export_to_html_dir = arg_filepath_fix(args.export_to_html_dir)
+        args.export_to_txt_dir = arg_filepath_fix(args.export_to_txt_dir)
         lang_str = initializations()
         CherryTreeHandler(args, lang_str)
-    elif DBUS_OK:
+    elif __builtin__.DBUS_OK is True:
         try:
             # client
             remote_object = session_bus.get_object("com.giuspen.CherryTreeService", "/CherryTreeObject")
@@ -224,7 +251,7 @@ def main(args):
                 si = subprocess.STARTUPINFO()
                 si.dwFlags |= subprocess.STARTF_USESHOWWINDOW
                 sys.stderr = os.devnull
-                subprocess.check_output(["taskkill", "/f", "/im", "dbus-daemon.exe"], startupinfo=si)
+                subprocess.check_output(["TASKKILL", "/F", "/IM", "dbus-daemon.exe", "/T"], startupinfo=si)
     else:
         print "dbus fail, maybe a firewall problem, centralized instances disabled"
         lang_str = initializations()
